@@ -70,6 +70,52 @@ class Pipette(Instruction):
         })
 
 
+class MagneticTransfer(Instruction):
+
+    '''
+    A magnetic_transfer instruction is constructed as a list of lists of
+    groups, executed in order, where each group is a collect, release, dry,
+    incubate, or mix sub-operation.  These sub-operations control the behavior
+    of tips which can be magnetized, and a heating platform. Groups in the same
+    list of groups use the same tips.
+
+    collect:
+
+        Collects beads from the specified "object" by raising and lowering
+        magnetized tips repeatedly with an optional pause at well bottom.
+
+    release:
+
+        Release beads from unmagnetized tips by oscillating the tips
+        vertically into and out of the "object".
+
+    dry:
+
+        Dry beads on magnetized tips above and outside the "object".
+
+    incubate:
+
+        Incubate the "object".
+
+    mix:
+
+        Oscillate the tips into and out of the "object"
+
+    '''
+
+    HEAD_TYPE = ["96-deep", "96-pcr"]
+
+    def __init__(self, groups, head_type):
+        if head_type not in self.HEAD_TYPE:
+            raise ValueError(
+                "Specified `head_type` not: %s" % ", ".join(self.HEAD_TYPE))
+        super(MagneticTransfer, self).__init__({
+            "op": "magnetic_transfer",
+            "groups": groups,
+            "magnetic_head": head_type
+        })
+
+
 class Dispense(Instruction):
 
     """
@@ -150,18 +196,36 @@ class Spin(Instruction):
     acceleration : str
         Amount of acceleration to be applied to the container, expressed in
         units of "g" or "meter/second^2"
-    duration : str
+    duration : str, Unit
         Amount of time to apply acceleration.
+    flow_direction: str
+        Specifies the direction contents will tend toward with respect to
+        the container. Valid directions are "inward" and "outward", default
+        value is "inward".
+    spin_direction: list of strings
+        A list of "cw" (clockwise), "cww" (counterclockwise). For each
+        element in the list, the container will be spun in the stated
+        direction for the set "acceleration" and "duration". Default values
+        are derived from the "flow_direction". If "flow_direction" is
+        "outward", then "spin_direction" defaults to ["cw", "ccw"]. If
+        "flow_direction" is "inward", then "spin_direction" defaults to
+        ["cw"].
 
     """
 
-    def __init__(self, ref, acceleration, duration):
-        super(Spin, self).__init__({
+    def __init__(self, ref, acceleration, duration, flow_direction=None, spin_direction=None):
+        spin_json = {
             "op": "spin",
             "object": ref,
             "acceleration": acceleration,
             "duration": duration
-        })
+        }
+        if flow_direction is not None:
+            spin_json["flow_direction"] = flow_direction
+        if spin_direction is not None:
+            spin_json["spin_direction"] = spin_direction
+
+        super(Spin, self).__init__(spin_json)
 
 
 class Thermocycle(Instruction):
@@ -301,8 +365,10 @@ class Thermocycle(Instruction):
         dye_names = reduce(lambda x, y: x.union(y),
                            [set(well_map[k]) for k in well_map])
         if Thermocycle.find_invalid_dyes(dye_names):
-            raise ValueError("thermocycle instruction supplied the following "
-                             "invalid dyes: %s" % ", ".join(Thermocycle.find_invalid_dyes(dye_names)))
+            invalid_dyes = ", ".join(Thermocycle.find_invalid_dyes(dye_names))
+            raise ValueError("thermocycle instruction supplied the following invalid dyes: {}"
+                             .format(invalid_dyes))
+
         dye_map = {dye: [] for dye in dye_names}
         for well in well_map:
             dyes = well_map[well]
@@ -349,6 +415,54 @@ class Incubate(Instruction):
         })
 
 
+class IlluminaSeq(Instruction):
+
+    """
+    Load aliquots into specified lanes for Illumina sequencing.
+    The specified aliquots should already contain the appropriate mix for
+    sequencing and require a library concentration reported in
+    ng/uL.
+
+    Parameters
+    ----------
+    flowcell : str
+      Flowcell designation: "SR" or " "PE"
+    lanes : list of dicts
+
+        .. code-block:: json
+
+          "lanes": [{
+                "object": aliquot, Well,
+                "library_concentration": decimal, // ng/uL
+              },
+              {...}]
+
+    sequencer : str
+      Sequencer designation: "miseq", "hiseq" or "nextseq"
+    mode : str
+      Mode designation: "rapid", "mid" or "high"
+    index : str
+      Index designation: "single", "dual" or "none"
+    library_size: integer
+        Library size expressed as an integer of basepairs
+    dataref : str
+      Name of sequencing dataset that will be returned.
+
+    """
+
+    def __init__(self, flowcell, lanes, sequencer, mode, index, library_size, dataref):
+        super(IlluminaSeq, self).__init__({
+            "op": "illumina_sequence",
+            "flowcell": flowcell,
+            "lanes": lanes,
+            "sequencer": sequencer,
+            "mode": mode,
+            "index": index,
+            "library_size": library_size,
+            "dataref": dataref
+        })
+
+
 class SangerSeq(Instruction):
 
     """
@@ -414,6 +528,55 @@ class GelSeparate(Instruction):
         })
 
 
+class GelPurify(Instruction):
+
+    """
+    Separate nucleic acids on an agarose gel and purify.
+
+    Parameters
+    ----------
+    wells : list, WellGroup
+        WellGroup of wells to be purified
+    volume : str, Unit
+        Volume of sample required for analysis
+    dataref : str
+        Name of this specific dataset of measurements
+    matrix : str
+        Agarose concentration and number of wells on gel used for separation
+    ladder : str
+        Size range of ladder to be used to compare band size to
+    dataref : str
+        Name of dataset containing fragment sizes returned
+    extract: list of Dictionary
+
+        .. code-block:: json
+
+          "extract": [{
+                "elution_volume": volume,
+                "elution_buffer": string, "water" | "TE",
+                "lane": int,
+                "band_size_range": {
+                  "min_bp": int,
+                  "max_bp": int,
+                },
+                "destination": well
+              },
+              {...}]
+
+    """
+
+    def __init__(self, wells, volume, matrix, ladder, dataref, extract):
+        super(GelPurify, self).__init__({
+            "op": "gel_purify",
+            "objects": wells,
+            "volume": volume,
+            "matrix": matrix,
+            "ladder": ladder,
+            "dataref": dataref,
+            "extract": extract
+        })
+
+
 class Absorbance(Instruction):
 
     """
@@ -432,18 +595,35 @@ class Absorbance(Instruction):
     dataref : str
         name of this specific dataset of measured absorbances
     flashes : int, optional
+    incubate_before: dict, optional
+        incubation prior to reading if desired
+        shaking: dict, optional
+            shake parameters if desired
+                amplitude: str, Unit
+                    amplitude of shaking between 1 and 6:millimeter
+                orbital: bool
+                    True for oribital and False for linear shaking
+        duration: str, Unit, optional
+            time prior to plate reading
+    temperature: str, Unit, optional
+        set temperature to heat plate reading chamber
 
     """
 
-    def __init__(self, ref, wells, wavelength, dataref, flashes=25):
-        super(Absorbance, self).__init__({
-            "op": "absorbance",
-            "object": ref,
-            "wells": wells,
-            "wavelength": wavelength,
-            "num_flashes": flashes,
-            "dataref": dataref
-        })
+    def __init__(self, ref, wells, wavelength, dataref, flashes=25,
+                 incubate_before=None, temperature=None):
+        json_dict = {"op": "absorbance",
+                     "object": ref,
+                     "wells": wells,
+                     "wavelength": wavelength,
+                     "num_flashes": flashes,
+                     "dataref": dataref}
+        if incubate_before:
+            json_dict["incubate_before"] = incubate_before
+        if temperature:
+            json_dict["temperature"] = temperature
+
+        super(Absorbance, self).__init__(json_dict)
 
 
 class Fluorescence(Instruction):
@@ -466,19 +646,41 @@ class Fluorescence(Instruction):
     dataref : str
         name of this specific dataset of measured absorbances
     flashes : int, optional
+    incubate_before: dict, optional
+        incubation prior to reading if desired
+        shaking: dict, optional
+            shake parameters if desired
+                amplitude: str, Unit
+                    amplitude of shaking between 1 and 6:millimeter
+                orbital: bool
+                    True for oribital and False for linear shaking
+        duration: str, Unit, optional
+            time prior to plate reading
+    temperature: str, Unit, optional
+        set temperature to heat plate reading chamber
+    gain: float, optional
+            float between 0 and 1, multiplier of maximum signal amplification
 
     """
 
-    def __init__(self, ref, wells, excitation, emission, dataref, flashes=25):
-        super(Fluorescence, self).__init__({
+    def __init__(self, ref, wells, excitation, emission, dataref, flashes=25,
+                 incubate_before=None, temperature=None, gain=None):
+        json_dict = {
             "op": "fluorescence",
             "object": ref,
             "wells": wells,
             "excitation": excitation,
             "emission": emission,
             "num_flashes": flashes,
-            "dataref": dataref
-        })
+            "dataref": dataref}
+        if incubate_before:
+            json_dict["incubate_before"] = incubate_before
+        if temperature:
+            json_dict["temperature"] = temperature
+        if gain is not None:
+            json_dict["gain"] = gain
+
+        super(Fluorescence, self).__init__(json_dict)
 
 
 class Luminescence(Instruction):
@@ -492,16 +694,34 @@ class Luminescence(Instruction):
     wells : list, WellGroup
         WellGroup or list of wells to be measured
     dataref : str
+    incubate_before: dict, optional
+        incubation prior to reading if desired
+        shaking: dict, optional
+            shake parameters if desired
+                amplitude: str, Unit
+                    amplitude of shaking between 1 and 6:millimeter
+                orbital: bool
+                    True for oribital and False for linear shaking
+        duration: str, Unit, optional
+            time prior to plate reading
+    temperature: str, Unit, optional
+        set temperature to heat plate reading chamber
 
     """
 
-    def __init__(self, ref, wells, dataref):
-        super(Luminescence, self).__init__({
+    def __init__(self, ref, wells, dataref, incubate_before=None, temperature=None):
+        json_dict = {
             "op": "luminescence",
             "object": ref,
             "wells": wells,
             "dataref": dataref
-        })
+        }
+        if incubate_before:
+            json_dict["incubate_before"] = incubate_before
+        if temperature:
+            json_dict["temperature"] = temperature
+
+        super(Luminescence, self).__init__(json_dict)
 
 
 class Seal(Instruction):
@@ -942,3 +1162,70 @@ class Stamp(Instruction):
             "op": "stamp",
             "groups": groups
         })
+
+
+class MeasureConcentration(Instruction):
+
+    """
+    Measure the concentration of DNA, ssDNA, RNA or Protein in the specified
+    volume of the source aliquots.
+
+    Parameters
+    ----------
+    wells : list, WellGroup
+        WellGroup of wells to be measured
+    volume : str, Unit
+        Volume of sample required for analysis
+    dataref : str
+        Name of this specific dataset of measurements
+    measurement : str
+        Class of material to be measured. One of ["DNA", "ssDNA", "RNA",
+        "protein"].
+
+    """
+
+    def __init__(self, wells, volume, dataref, measurement):
+        json_dict = {"op": "measure_concentration",
+                     "object": wells,
+                     "volume": volume,
+                     "dataref": dataref,
+                     "measurement": measurement}
+        super(MeasureConcentration, self).__init__(json_dict)
+
+
+class MeasureMass(Instruction):
+    """
+    Measure the mass of containers
+
+    Parameters
+    ----------
+    refs : list of containers
+        list of containers
+    dataref: str
+        Name of the data for the measurement
+    """
+
+    def __init__(self, refs, dataref):
+        json_dict = {"op": "measure_mass",
+                     "object": refs,
+                     "dataref": dataref}
+        super(MeasureMass, self).__init__(json_dict)
+
+
+class MeasureVolume(Instruction):
+    """
+    Measure the mass of containers
+
+    Parameters
+    ----------
+    refs : list of containers
+        list of containers
+    dataref: str
+        Name of the data for the measurement
+    """
+
+    def __init__(self, wells, dataref):
+        json_dict = {"op": "measure_volume",
+                     "object": wells,
+                     "dataref": dataref}
+        super(MeasureVolume, self).__init__(json_dict)
